@@ -21,6 +21,12 @@
 (defconstant LAMBDA-PARAMETERS-LIMIT 4294967296)
 (defconstant CALL-ARGUMENTS-LIMIT 4294967296)
 (defconstant MULTIPLE-VALUES-LIMIT 4294967296)
+(defconstant ARRAY-RANK-LIMIT 4294967296)
+(defconstant ARRAY-DIMENSION-LIMIT 4294967296)
+(defconstant ARRAY-TOTAL-SIZE-LIMIT 4294967296)
+
+
+(defun identity (x) x)
 
 
 (defconstant PI 3.141592653589793L0)
@@ -45,6 +51,18 @@
 (defun zerop (num)
   (= num 0))
 
+(defun mod (num div)
+  (nth-value 1 (floor num div)))
+
+(defun rem (num div)
+  (nth-value 1 (truncate num div)))
+
+(defun evenp (num)
+  (= 0 (mod num 2)))
+
+(defun oddp (num)
+  (= 1 (mod num 2)))
+
 (defun isqrt (num)
   (values (floor (sqrt num))))
 
@@ -94,6 +112,9 @@
 (defun cddddr (list) (nthcdr 4 list))
 
 
+(defun endp (l) (null l))
+
+
 (defmacro when (clause &body body)
   `(if ,clause (progn ,@body)))
 
@@ -134,15 +155,111 @@
 	     (cond ,@rest)))))
 
 
+(defmacro return (&optional val)
+  `(return-from nil (values-list (multiple-value-list ,val))))
+
+
+(defun every (pred &rest sequences)
+  (let ((n (apply 'min (mapcar #'length sequences))))
+    (dotimes (i n)
+      (let ((args (mapcar (lambda (s) (elt s i)) sequences)))
+	(if (not (apply pred args))
+	    (return-from every nil)))))
+  t)
+
+
+(defun some (pred &rest sequences)
+  (not (apply 'every (complement pred) sequences)))
+
+
+(defun notany (pred &rest sequences)
+  (not (apply 'some pred sequences)))
+
+
+(defun notevery (pred &rest sequences)
+  (not (apply 'every pred sequences)))
+
+
+(defun member (obj list)
+  (let ((subl list))
+    (dotimes (i (length list))
+      (if (eql obj (car subl))
+	  (return-from member subl))
+      (setq subl (cdr subl)))))
+
+
+(defun member-if (pred list)
+  (let ((subl list))
+    (dotimes (i (length list))
+      (if (funcall pred (car subl))
+	  (return-from member-if subl))
+      (setq subl (cdr subl)))))
+
+
+(defun member-if-not (pred list)
+  (member-if (complement pred) list))
+
+
+(defun count (obj seq)
+  (let ((ret 0))
+    (dotimes (i (length seq))
+      (if (eql (elt seq i) obj)
+	  (setq ret (+ 1 ret))))
+    ret))
+
+
+(defun count-if (pred seq)
+  (let ((ret 0))
+    (dotimes (i (length seq))
+      (if (funcall pred (elt seq i))
+	  (setq ret (+ 1 ret))))
+    ret))
+
+
+(defun count-if-not (pred seq)
+  (count-if (complement pred) seq))
+
+
+(defun remove (obj seq)
+  (remove-if (lambda (ob) (eql ob obj)) seq))
+
+
+(defun remove-if-not (pred seq)
+  (remove-if (complement pred) seq))
+
+
+(defun delete (obj seq)
+  (remove-if (lambda (ob) (eql ob obj)) seq))
+
+
+(defun delete-if (pred seq)
+  (remove-if pred seq))
+
+
+(defun delete-if-not (pred seq)
+  (remove-if (complement pred) seq))
+
+
+(defun nreverse (seq)
+  (reverse seq))
+
 
 (defun array-dimension (array axis)
   (nth axis (array-dimensions array)))
 
 
+(defun array-total-size (array)
+  (apply #'* (array-dimensions array)))
+
+
 (defun char (str ind)
   (aref str ind))
 
 
+(defun string/= (str1 str2)
+  (not (string= str1 str2)))
+
+
 
 (defun consp (obj)
   (typep obj 'cons))
@@ -201,6 +318,11 @@
 
 
 
+(defun complement (func)
+  (lambda (&rest args) (not (apply func args))))
+
+
+
 (defun mapc (fun &rest lists)
   (apply #'mapcar (cons fun lists))
   (car lists))
